/*
 * $Id: xhbwith.c 8924 2008-07-03 16:37:01Z vszakats $
 */

/*
 * Harbour Project source code:
 *    xHarbour compatible WITH OBJECT internal functions
 *
 * Copyright 2007 Przemyslaw Czerpak <druzus / at / priv.onet.pl>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */

#include "hbvmopt.h"
#include "hbapi.h"
#include "hbapiitm.h"
#include "hbapierr.h"
#include "hbstack.h"

static PHB_ITEM hb_vmWithObjectItem( LONG lLevel )
{
   LONG lOffset = hb_stackWithObjectOffset();

   while( lOffset && lLevel > 0 )
   {
      LONG * plOffset = ( LONG * ) hb_itemGetPtr( hb_stackItem( lOffset + 1 ) );
      if( !plOffset )
         break;
      --lLevel;
      lOffset = *plOffset;
   }

   return ( lOffset && !lLevel ) ? hb_stackItem( lOffset ) : NULL;
}

static LONG hb_vmWithObjectCount( void )
{
   LONG lOffset = hb_stackWithObjectOffset(), lCount = 0;

   while( lOffset )
   {
      LONG * plOffset = ( LONG * ) hb_itemGetPtr( hb_stackItem( lOffset + 1 ) );
      if( !plOffset )
         break;
      ++lCount;
      lOffset = *plOffset;
   }

   return lCount;
}



HB_FUNC( HB_QWITH )
{
   hb_itemReturn( hb_vmWithObjectItem( hb_parnl( 1 ) ) );
}

HB_FUNC( HB_WITHOBJECTCOUNTER )
{
   hb_retnl( hb_vmWithObjectCount() );
}

HB_FUNC( HB_RESETWITH )
{
   PHB_ITEM pItem = hb_vmWithObjectItem( 0 );

   if( hb_pcount() >= 1 && pItem )
      hb_itemMove( pItem, hb_stackItemFromBase( 1 ) );
   else
      hb_errRT_BASE( EG_ARG, 1607, NULL, HB_ERR_FUNCNAME, 0, NULL );
}
